@@ -77,7 +77,10 @@ int segy_field_forall( segy_file*,
 int segy_trace_bsize( int samples );
 /* byte-offset of the first trace header. */
 long segy_trace0( const char* binheader );
-/* number of traces in this file */
+/*
+ * number of traces in this file.
+ * if this function fails, the input argument is not modified.
+ */
 int segy_traces( segy_file*, int*, long trace0, int trace_bsize );
 
 int segy_sample_indices( segy_file*,
@@ -87,6 +90,8 @@ int segy_sample_indices( segy_file*,
                          float* buf );
 
 /* text header operations */
+/* buf in all read functions should be minimum segy_textheader_size() in size */
+/* all read_textheader function outputs are zero-terminated C strings */
 int segy_read_textheader( segy_file*, char *buf);
 int segy_textheader_size( void );
 /*
@@ -248,6 +253,9 @@ int segy_write_line( segy_file* fp,
  *
  * `offsets` is the number of offsets in the file and be found with
  * `segy_offsets`.
+ *
+ * If the file has only 1 trace (or, for pre-stack files, 1-trace-per-offset),
+ * segyio considers this as 1 line in each direction.
  */
 int segy_count_lines( segy_file*,
                       int field,
@@ -535,6 +543,8 @@ typedef enum {
     SEGY_INVALID_ARGS,
     SEGY_MMAP_ERROR,
     SEGY_MMAP_INVALID,
+    SEGY_READONLY,
+    SEGY_NOTFOUND,
 } SEGY_ERROR;
 
 #ifdef __cplusplus
