@@ -33,7 +33,7 @@
 #include <segyio/segy.h>
 #include <segyio/util.h>
 
-static unsigned char a2e[256] = {
+static const unsigned char a2e[256] = {
     0,  1,  2,  3,  55, 45, 46, 47, 22, 5,  37, 11, 12, 13, 14, 15,
     16, 17, 18, 19, 60, 61, 50, 38, 24, 25, 63, 39, 28, 29, 30, 31,
     64, 79, 127,123,91, 108,80, 125,77, 93, 92, 78, 107,96, 75, 97,
@@ -52,7 +52,7 @@ static unsigned char a2e[256] = {
     220,221,222,223,234,235,236,237,238,239,250,251,252,253,254,255
 };
 
-static unsigned char e2a[256] = {
+static const unsigned char e2a[256] = {
     0,  1,  2,  3,  156,9,  134,127,151,141,142, 11,12, 13, 14, 15,
     16, 17, 18, 19, 157,133,8,  135,24, 25, 146,143,28, 29, 30, 31,
     128,129,130,131,132,10, 23, 27, 136,137,138,139,140,5,  6,  7,
@@ -71,18 +71,33 @@ static unsigned char e2a[256] = {
     48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 250,251,252,253,254,255
 };
 
-void ebcdic2ascii( const char* ebcdic, char* ascii ) {
-    while( *ebcdic != '\0' )
-        *ascii++ = (char)e2a[ (unsigned char) *ebcdic++ ];
+static int encode( char* dst,
+                   const char* src,
+                   const unsigned char* conv,
+                   size_t n ) {
+    for( size_t i = 0; i < n; ++i )
+        dst[ i ] = (char)conv[ (unsigned char) src[ i ] ];
 
-    *ascii = '\0';
+    return SEGY_OK;
+}
+
+/*
+ * DEPRECATED
+ * ebcdic2ascii and ascii2ebcdic are deprecated in favour of the length-aware
+ * encode. They will be removed in segyio2. These functions were never public
+ * (in the sense they're not available in headers), but currently have external
+ * linkage.
+ */
+void ebcdic2ascii( const char* ebcdic, char* ascii ) {
+    size_t len = strlen( ebcdic );
+    encode( ascii, ebcdic, e2a, len );
+    ascii[ len ] = '\0';
 }
 
 void ascii2ebcdic( const char* ascii, char* ebcdic ) {
-    while (*ascii != '\0')
-        *ebcdic++ = (char)a2e[(unsigned char) *ascii++];
-
-    *ebcdic = '\0';
+    size_t len = strlen( ascii );
+    encode( ebcdic, ascii, a2e, len );
+    ebcdic[ len ] = '\0';
 }
 
 #define IEEEMAX 0x7FFFFFFF
@@ -294,6 +309,12 @@ static int bfield_size[] = {
  */
 #ifdef HAVE_SYS_STAT_H
 static int file_size( FILE* fp, long long* size ) {
+
+    /*
+     * the file size will be unaccurate unless userland buffers are flushed if
+     * the file is new or appended to
+     */
+    if( fflush( fp ) != 0 ) return SEGY_FWRITE_ERROR;
 #ifdef HAVE_FSTATI64
     // this means we're on windows where fstat is unreliable for filesizes >2G
     // because long is only 4 bytes
@@ -314,12 +335,15 @@ static int file_size( FILE* fp, long long* size ) {
  * addr is NULL if mmap is not found under compilation or if the file is
  * not requested mmap'd. If so, the fallback code path of FILE* is taken
  */
+
+#define MODEBUF_SIZE 5
 struct segy_file_handle {
     void* addr;
     void* cur;
     FILE* fp;
     size_t fsize;
-    char mode[ 4 ];
+    char mode[ MODEBUF_SIZE ];
+    int writable;
 };
 
 segy_file* segy_open( const char* path, const char* mode ) {
@@ -328,7 +352,7 @@ segy_file* segy_open( const char* path, const char* mode ) {
 
     // append a 'b' if it is not passed by the user; not a problem on unix, but
     // windows and other platforms fail without it
-    char binary_mode[ 4 ] = { 0 };
+    char binary_mode[ MODEBUF_SIZE ] = { 0 };
     strncpy( binary_mode, mode, 3 );
 
     size_t mode_len = strlen( binary_mode );
@@ -353,6 +377,9 @@ segy_file* segy_open( const char* path, const char* mode ) {
     file->fp = fp;
     strcpy( file->mode, binary_mode );
 
+    bool rw = strstr( file->mode, "+" ) || strstr( file->mode, "w" );
+    if( rw ) file->writable = 1;
+
     return file;
 }
 
@@ -369,8 +396,7 @@ int segy_mmap( segy_file* fp ) {
 
     if( err != 0 ) return SEGY_FSEEK_ERROR;
 
-    bool rw = strstr( fp->mode, "+" ) || strstr( fp->mode, "w" );
-    const int prot =  rw ? PROT_READ | PROT_WRITE : PROT_READ;
+    const int prot = fp->writable ? PROT_READ | PROT_WRITE : PROT_READ;
 
     int fd = fileno( fp->fp );
     void* addr = mmap( NULL, fsize, prot, MAP_SHARED, fd, 0 );
@@ -380,22 +406,27 @@ int segy_mmap( segy_file* fp ) {
 
     fp->addr = fp->cur = addr;
     fp->fsize = fsize;
+
+    fclose(fp->fp);
+
     return SEGY_OK;
 #endif //HAVE_MMAP
 }
 
 int segy_flush( segy_file* fp, bool async ) {
-    int syncerr = 0;
+
+    // flush is a no-op for read-only files
+    if( !fp->writable ) return SEGY_OK;
 
 #ifdef HAVE_MMAP
     if( fp->addr ) {
         int flag = async ? MS_ASYNC : MS_SYNC;
-        syncerr = msync( fp->addr, fp->fsize, flag );
+        int syncerr = msync( fp->addr, fp->fsize, flag );
+        if( syncerr != 0 ) return syncerr;
+        return SEGY_OK;
     }
 #endif //HAVE_MMAP
 
-    if( syncerr != 0 ) return syncerr;
-
     int flusherr = fflush( fp->fp );
 
     if( flusherr != 0 ) return SEGY_FWRITE_ERROR;
@@ -427,6 +458,9 @@ int segy_close( segy_file* fp ) {
     if( err != 0 )
         err = SEGY_MMAP_ERROR;
 
+    free( fp );
+    return err;
+
 no_mmap:
 #endif //HAVE_MMAP
 
@@ -517,6 +551,8 @@ int segy_set_bfield( char* binheader, int field, int val ) {
 }
 
 static int slicelength( int start, int stop, int step ) {
+    if( step == 0 ) return 0;
+
     if( ( step < 0 && stop >= start ) ||
         ( step > 0 && start >= stop ) ) return 0;
 
@@ -550,6 +586,7 @@ int segy_field_forall( segy_file* fp,
     err = segy_seek( fp, end, trace0, trace_bsize );
     if( err != SEGY_OK ) return err;
 
+#ifdef HAVE_MMAP
     if( fp->addr ) {
         for( int i = start; slicelen > 0; i += step, ++buf, --slicelen ) {
             segy_seek( fp, i, trace0, trace_bsize );
@@ -559,6 +596,7 @@ int segy_field_forall( segy_file* fp,
 
         return SEGY_OK;
     }
+#endif //HAVE_MMAP
 
     /*
      * non-mmap path. Doing multiple freads is slow, so instead the *actual*
@@ -583,11 +621,50 @@ int segy_field_forall( segy_file* fp,
     return SEGY_OK;
 }
 
+/*
+ * memread/memwrite are small utilities to give reading/writing to
+ * memory-mapped files fread/fwrite like behaviour and fail if going outside
+ * the file. Returns SEGY_FREAD/WRITE_ERROR, so that functions that are
+ * read-and-return can just return this function's result
+ */
+static int memread( void* dest, const segy_file* fp, const void* src, size_t n ) {
+    const void* begin = fp->addr;
+    const void* end = (const char*)fp->addr + fp->fsize;
+    const void* srcend = (const char*)src + n;
+
+    if( src < begin || src > end || srcend > end )
+        return SEGY_FREAD_ERROR;
+
+    memcpy( dest, src, n );
+    return SEGY_OK;
+}
+
+static int memwrite( segy_file* fp, void* dest, const void* src, size_t n ) {
+    const void* begin = fp->addr;
+    const void* end = (const char*)fp->addr + fp->fsize;
+    const void* destend = (const char*)dest + n;
+
+    if( dest < begin || dest > end || destend > end )
+        return SEGY_FWRITE_ERROR;
+
+    memcpy( dest, src, n );
+    return SEGY_OK;
+}
+
 int segy_binheader( segy_file* fp, char* buf ) {
     if(fp == NULL) {
         return SEGY_INVALID_ARGS;
     }
 
+#ifdef HAVE_MMAP
+    if( fp->addr ) {
+        return memread( buf,
+                        fp,
+                        (char*)fp->addr + SEGY_TEXT_HEADER_SIZE,
+                        SEGY_BINARY_HEADER_SIZE );
+    }
+#endif //HAVE_MMAP
+
     const int err = fseek( fp->fp, SEGY_TEXT_HEADER_SIZE, SEEK_SET );
     if( err != 0 ) return SEGY_FSEEK_ERROR;
 
@@ -599,9 +676,16 @@ int segy_binheader( segy_file* fp, char* buf ) {
 }
 
 int segy_write_binheader( segy_file* fp, const char* buf ) {
-    if(fp == NULL) {
-        return SEGY_INVALID_ARGS;
+    if( !fp->writable ) return SEGY_READONLY;
+
+#ifdef HAVE_MMAP
+    if( fp->addr ) {
+        return memwrite( fp,
+                        (char*)fp->addr + SEGY_TEXT_HEADER_SIZE,
+                         buf,
+                         SEGY_BINARY_HEADER_SIZE );
     }
+#endif //HAVE_MMAP
 
     const int err = fseek( fp->fp, SEGY_TEXT_HEADER_SIZE, SEEK_SET );
     if( err != 0 ) return SEGY_FSEEK_ERROR;
@@ -614,13 +698,13 @@ int segy_write_binheader( segy_file* fp, const char* buf ) {
 }
 
 int segy_format( const char* binheader ) {
-    int format;
+    int32_t format = 0;
     segy_get_bfield( binheader, SEGY_BIN_FORMAT, &format );
     return format;
 }
 
 int segy_samples( const char* binheader ) {
-    int32_t samples;
+    int32_t samples = 0;
     segy_get_bfield( binheader, SEGY_BIN_SAMPLES, &samples );
     return samples;
 }
@@ -632,7 +716,7 @@ int segy_trace_bsize( int samples ) {
 }
 
 long segy_trace0( const char* binheader ) {
-    int extra_headers;
+    int extra_headers = 0;
     segy_get_bfield( binheader, SEGY_BIN_EXT_HEADERS, &extra_headers );
 
     return SEGY_TEXT_HEADER_SIZE + SEGY_BINARY_HEADER_SIZE +
@@ -647,12 +731,17 @@ int segy_seek( segy_file* fp,
     trace_bsize += SEGY_TRACE_HEADER_SIZE;
     long long pos = (long long)trace0 + (trace * (long long)trace_bsize);
 
+#ifdef HAVE_MMAP
     if( fp->addr ) {
-        if( (size_t)pos >= fp->fsize ) return SEGY_FSEEK_ERROR;
-
+        /*
+         * mmap fseek doesn't fail (it's just a pointer readjustment) and won't
+         * set errno, in order to keep its behaviour consistent with fseek,
+         * which can easily reposition itself past the end-of-file
+         */
         fp->cur = (char*)fp->addr + pos;
         return SEGY_OK;
     }
+#endif //HAVE_MMAP
 
     int err;
 #if LONG_MAX == LLONG_MAX
@@ -690,10 +779,7 @@ int segy_traceheader( segy_file* fp,
     const int err = segy_seek( fp, traceno, trace0, trace_bsize );
     if( err != 0 ) return err;
 
-    if( fp->addr ) {
-        memcpy( buf, fp->cur, SEGY_TRACE_HEADER_SIZE );
-        return SEGY_OK;
-    }
+    if( fp->addr ) return memread( buf, fp, fp->cur, SEGY_TRACE_HEADER_SIZE );
 
     const size_t readc = fread( buf, 1, SEGY_TRACE_HEADER_SIZE, fp->fp );
 
@@ -708,14 +794,12 @@ int segy_write_traceheader( segy_file* fp,
                             const char* buf,
                             long trace0,
                             int trace_bsize ) {
+    if( !fp->writable ) return SEGY_READONLY;
 
     const int err = segy_seek( fp, traceno, trace0, trace_bsize );
     if( err != 0 ) return err;
 
-    if( fp->addr ) {
-        memcpy( fp->cur, buf, SEGY_TRACE_HEADER_SIZE );
-        return SEGY_OK;
-    }
+    if( fp->addr ) return memwrite( fp, fp->cur, buf, SEGY_TRACE_HEADER_SIZE );
 
     const size_t writec = fwrite( buf, 1, SEGY_TRACE_HEADER_SIZE, fp->fp );
 
@@ -736,9 +820,15 @@ int segy_traces( segy_file* fp,
                  long trace0,
                  int trace_bsize ) {
 
+    if( trace0 < 0 ) return SEGY_INVALID_ARGS;
+
     long long size;
-    int err = file_size( fp->fp, &size );
-    if( err != 0 ) return err;
+    if( fp->addr )
+        size = fp->fsize;
+    else{
+        int err = file_size( fp->fp, &size );
+        if( err != 0 ) return err;
+    }
 
     if( trace0 > size ) return SEGY_INVALID_ARGS;
 
@@ -773,8 +863,8 @@ int segy_sample_interval( segy_file* fp, float fallback, float* dt ) {
         return err;
     }
 
-    int bindt;
-    int trdt;
+    int bindt = 0;
+    int trdt = 0;
 
     segy_get_bfield( bin_header, SEGY_BIN_INTERVAL, &bindt );
     segy_get_field( trace_header, SEGY_TR_SAMPLE_INTER, &trdt );
@@ -854,7 +944,7 @@ int segy_sorting( segy_file* fp,
     if( field_size[ il ] == 0 || field_size[ xl ] == 0 )
         return SEGY_INVALID_FIELD;
 
-    int il0, xl0, il1, xl1, off0, off1;
+    int il0 = 0, xl0 = 0, il1 = 0, xl1 = 0, off0 = 0, off1 = 0;
 
     segy_get_field( traceheader, il, &il0 );
     segy_get_field( traceheader, xl, &xl0 );
@@ -883,7 +973,7 @@ int segy_sorting( segy_file* fp,
     err = segy_traceheader( fp, traces - 1, traceheader, trace0, trace_bsize );
     if( err != SEGY_OK ) return err;
 
-    int il_last, xl_last;
+    int il_last = 0, xl_last = 0;
     segy_get_field( traceheader, il, &il_last );
     segy_get_field( traceheader, xl, &xl_last );
 
@@ -910,7 +1000,7 @@ int segy_offsets( segy_file* fp,
                   long trace0,
                   int trace_bsize ) {
     int err;
-    int il0, il1, xl0, xl1;
+    int il0 = 0, il1 = 0, xl0 = 0, xl1 = 0;
     char header[ SEGY_TRACE_HEADER_SIZE ];
     int offsets = 0;
 
@@ -927,6 +1017,8 @@ int segy_offsets( segy_file* fp,
         return SEGY_INVALID_FIELD;
 
     err = segy_traceheader( fp, 0, header, trace0, trace_bsize );
+    if( err != 0 ) return SEGY_FREAD_ERROR;
+
     segy_get_field( header, il, &il0 );
     segy_get_field( header, xl, &xl0 );
 
@@ -990,6 +1082,7 @@ static int segy_line_indices( segy_file* fp,
 static int count_lines( segy_file* fp,
                         int field,
                         int offsets,
+                        int traces,
                         int* out,
                         long trace0,
                         int trace_bsize ) {
@@ -999,7 +1092,7 @@ static int count_lines( segy_file* fp,
     err = segy_traceheader( fp, 0, header, trace0, trace_bsize );
     if( err != 0 ) return err;
 
-    int first_lineno, first_offset, ln, off;
+    int first_lineno, first_offset, ln = 0, off = 0;
 
     err = segy_get_field( header, field, &first_lineno );
     if( err != 0 ) return err;
@@ -1011,6 +1104,8 @@ static int count_lines( segy_file* fp,
     int curr = offsets;
 
     while( true ) {
+        if( curr >= traces ) return SEGY_NOTFOUND;
+
         err = segy_traceheader( fp, curr, header, trace0, trace_bsize );
         if( err != 0 ) return err;
 
@@ -1035,13 +1130,27 @@ int segy_count_lines( segy_file* fp,
                       long trace0,
                       int trace_bsize ) {
 
-    int err;
-    int l2count;
-    err = count_lines( fp, field, offsets, &l2count, trace0, trace_bsize );
+    int traces;
+    int err = segy_traces( fp, &traces, trace0, trace_bsize );
     if( err != 0 ) return err;
 
-    int traces;
-    err = segy_traces( fp, &traces, trace0, trace_bsize );
+    /*
+     * handle the case where there's only one trace (per offset) in the file,
+     * and interpret is as a 1 line in each direction, with 1 trace (per
+     * offset).
+     */
+    if( traces == offsets ) {
+        *l1out = *l2out = 1;
+        return SEGY_OK;
+    }
+
+    int l2count;
+    err = count_lines( fp, field,
+                           offsets,
+                           traces,
+                           &l2count,
+                           trace0,
+                           trace_bsize );
     if( err != 0 ) return err;
 
     const int line_length = l2count * offsets;
@@ -1204,7 +1313,8 @@ int segy_readsubtr( segy_file* fp,
     if( step == 1 || step == -1 ) {
 
         if( fp->addr ) {
-            memcpy( buf, fp->cur, sizeof( float ) * elems );
+            err = memread( buf, fp, fp->cur, sizeof( float ) * elems );
+            if( err != SEGY_OK ) return err;
         } else {
             const size_t readc = fread( buf, sizeof( float ), elems, fp->fp );
             if( readc != elems ) return SEGY_FREAD_ERROR;
@@ -1272,6 +1382,8 @@ int segy_writesubtr( segy_file* fp,
                      long trace0,
                      int trace_bsize ) {
 
+    if( !fp->writable ) return SEGY_READONLY;
+
     int err = subtr_seek( fp, traceno, start, stop, trace0, trace_bsize );
     if( err != SEGY_OK ) return err;
 
@@ -1285,7 +1397,8 @@ int segy_writesubtr( segy_file* fp,
          * be handled by the stride-aware code path
          */
         if( fp->addr ) {
-            memcpy( fp->cur, buf, sizeof( float ) * elems );
+            err = memread( fp, fp->cur, buf, sizeof( float ) * elems );
+            if( err != SEGY_OK ) return err;
         } else {
             const size_t writec = fwrite( buf, sizeof( float ), elems, fp->fp );
             if( writec != elems ) return SEGY_FWRITE_ERROR;
@@ -1444,6 +1557,7 @@ int segy_write_line( segy_file* fp,
                      const float* buf,
                      long trace0,
                      int trace_bsize ) {
+    if( !fp->writable ) return SEGY_READONLY;
 
     assert( sizeof( float ) == sizeof( int32_t ) );
     assert( trace_bsize % 4 == 0 );
@@ -1523,34 +1637,52 @@ int segy_read_ext_textheader( segy_file* fp, int pos, char *buf) {
 
     const long offset = pos == -1 ? 0 :
                         SEGY_TEXT_HEADER_SIZE + SEGY_BINARY_HEADER_SIZE +
-                        ((pos-1) * SEGY_TEXT_HEADER_SIZE);
+                        (pos * SEGY_TEXT_HEADER_SIZE);
+
+#ifdef HAVE_MMAP
+    if ( fp->addr ) {
+        encode( buf, (char*)fp->addr + offset, e2a, SEGY_TEXT_HEADER_SIZE );
+        buf[ SEGY_TEXT_HEADER_SIZE ] = '\0';
+        return SEGY_OK;
+    }
+#endif //HAVE_MMAP
 
     int err = fseek( fp->fp, offset, SEEK_SET );
     if( err != 0 ) return SEGY_FSEEK_ERROR;
 
-    char localbuf[ SEGY_TEXT_HEADER_SIZE + 1 ];
+    char localbuf[ SEGY_TEXT_HEADER_SIZE + 1 ] = { 0 };
     const size_t read = fread( localbuf, 1, SEGY_TEXT_HEADER_SIZE, fp->fp );
     if( read != SEGY_TEXT_HEADER_SIZE ) return SEGY_FREAD_ERROR;
 
-    localbuf[ SEGY_TEXT_HEADER_SIZE ] = '\0';
-    ebcdic2ascii( localbuf, buf );
+    encode( buf, localbuf, e2a, SEGY_TEXT_HEADER_SIZE );
     return SEGY_OK;
 }
 
 int segy_write_textheader( segy_file* fp, int pos, const char* buf ) {
+    if( !fp->writable ) return SEGY_READONLY;
+
     int err;
-    char mbuf[ SEGY_TEXT_HEADER_SIZE + 1 ];
+    char mbuf[ SEGY_TEXT_HEADER_SIZE ];
 
     if( pos < 0 ) return SEGY_INVALID_ARGS;
 
-    // TODO: reconsider API, allow non-zero terminated strings
-    ascii2ebcdic( buf, mbuf );
+    err = encode( mbuf, buf, a2e, SEGY_TEXT_HEADER_SIZE );
+    if( err != 0 ) return err;
 
     const long offset = pos == 0
                       ? 0
                       : SEGY_TEXT_HEADER_SIZE + SEGY_BINARY_HEADER_SIZE +
                         ((pos-1) * SEGY_TEXT_HEADER_SIZE);
 
+#ifdef HAVE_MMAP
+    if( fp->addr ) {
+        return memwrite( fp,
+                        (char*)fp->addr + offset,
+                        mbuf,
+                        SEGY_TEXT_HEADER_SIZE );
+    }
+#endif //HAVE_MMAP
+
     err = fseek( fp->fp, offset, SEEK_SET );
     if( err != 0 ) return SEGY_FSEEK_ERROR;
 
